// -*- C++ -*-
/*!
 * @file  RaspWEBCamera.cpp
 * @brief get imageDate in WEBCamera
 * @date $Date$
 *
 * $Id$
 */

#include "RaspWEBCamera.h"

// Module specification
// <rtc-template block="module_spec">
static const char* raspwebcamera_spec[] =
  {
    "implementation_id", "RaspWEBCamera",
    "type_name",         "RaspWEBCamera",
    "description",       "get imageDate in WEBCamera",
    "version",           "1.0.0",
    "vendor",            "Aize",
    "category",          "Camera",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.height", "240",
    "conf.default.width", "320",

    // Widget
    "conf.__widget__.height", "text",
    "conf.__widget__.width", "text",
    // Constraints

    "conf.__type__.height", "int",
    "conf.__type__.width", "int",

    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
RaspWEBCamera::RaspWEBCamera(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_WebCameraImageOut("WebCameraImageOut", m_WebCameraImage)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
RaspWEBCamera::~RaspWEBCamera()
{
}



RTC::ReturnCode_t RaspWEBCamera::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  addOutPort("WebCameraImageOut", m_WebCameraImageOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("height", m_height, "240");
  bindParameter("width", m_width, "320");
  // </rtc-template>
  height = m_height;
  width = m_width;
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RaspWEBCamera::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RaspWEBCamera::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RaspWEBCamera::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t RaspWEBCamera::onActivated(RTC::UniqueId ec_id)
{
	std::cout << "Active" << std::endl;
	// Jڑ
	cap.open(0);
	if (cap.isOpened())std::cout << "USB Camera Connect" << std::endl;
	else std::cout << "USB Camera UnConnect" << std::endl;
	// ʃTCYݒ
	cap.set(CV_CAP_PROP_FRAME_WIDTH, m_width);
	cap.set(CV_CAP_PROP_FRAME_HEIGHT, m_height);

	return RTC::RTC_OK;
}



RTC::ReturnCode_t RaspWEBCamera::onDeactivated(RTC::UniqueId ec_id)
{
	std::cout << "Deactive" << std::endl;
	// Jp̊J
	cap.release();
	return RTC::RTC_OK;
}



RTC::ReturnCode_t RaspWEBCamera::onExecute(RTC::UniqueId ec_id)
{
	if (cap.isOpened()){

		if (width != m_width || height != m_height)
		{
			// ʃTCYĐݒ
			cap.set(CV_CAP_PROP_FRAME_WIDTH, m_width);
			cap.set(CV_CAP_PROP_FRAME_HEIGHT, m_height);

			std::cout << "Width:" << m_width << " Height:" << m_height << std::endl;
		}
		//摜擾
		cap.read(mat_image);

		IplImage frame = mat_image;

		int len = frame.nChannels * frame.width * frame.height;
		// ʃTCY
		m_WebCameraImage.pixels.length(len);
		m_WebCameraImage.width = frame.width;
		m_WebCameraImage.height = frame.height;
		//摜f[^OutPortɃRs[
		memcpy((void *)&(m_WebCameraImage.pixels[0]), frame.imageData, len);
		//摜f[^o
		m_WebCameraImageOut.write();

		//RtBM[V̒lۑ
		height = m_height;
		width = m_width;
	}
	return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t RaspWEBCamera::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RaspWEBCamera::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RaspWEBCamera::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RaspWEBCamera::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RaspWEBCamera::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void RaspWEBCameraInit(RTC::Manager* manager)
  {
    coil::Properties profile(raspwebcamera_spec);
    manager->registerFactory(profile,
                             RTC::Create<RaspWEBCamera>,
                             RTC::Delete<RaspWEBCamera>);
  }
  
};


